from unittest import TestCase

from risc_generator import Premium

INCLUDE = "include"
EXCLUDE = "exclude"


class PremiumTest(TestCase):
    def test_givenASetOfProtection_whenPremium_thenSetProtections(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        actual_premium = Premium(**protection_premiums)
        self.assertIsInstance(actual_premium, Premium)

        self.assertIsNotNone(actual_premium.get("LiabilityPremium"))

        # + 1 for the total premium amount
        self.assertEqual(len(list(actual_premium.__dict__.items())), len(list(protection_premiums.items())) + 1)

    def test_givenASetOfProtection_whenGet_thenReturnPremium(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        actual_premium = Premium(**protection_premiums)
        self.assertIsInstance(actual_premium, Premium)

        for key, _ in protection_premiums.items():
            self.assertIsNotNone(actual_premium.get(key))

    def test_givenASetOfProtection_whenGetOnProtectionNotInSetOfProtection_thenReturnNone(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
        }

        actual_premium = Premium(**protection_premiums)
        self.assertIsInstance(actual_premium, Premium)

        protection_not_in_set_of_protections = [
            "FAQ20ACoveragePremium",
            "FAQ33CoveragePremium",
            "FAQ34CoveragePremium",
            "FAQ43CoveragePremium",
        ]
        for key in protection_not_in_set_of_protections:
            self.assertIsNone(actual_premium.get(key))

    def test_whenAProtection_getWithCamelCaseProtectionName_returnProtection(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
        }

        actual_premium = Premium(**protection_premiums)
        self.assertIsNotNone(actual_premium.get("LiabilityPremium"))



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq43_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 43 (A à F)\n"
        "Modification à l'indemnisation\n"
        "(Chapitre B)\n"
        "Le titre de l'avenant et les options applicables doivent être écrits à la section « Conditions "
        "particulières » du contrat d'assurance. Quant aux informations requises dans l'avenant, elles "
        "peuvent être écrites à cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N : \n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Prime d'assurance additionnelle à payer :\n"
        "\t Montants à payer : \n"
        "\t Date limite pour payer :\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant étend les garanties du chapitre B du contrat d'assurance en modifiant, selon "
        "les options applicables, l'article 2 de la section « Déclarer un sinistre et faire une "
        "réclamation ».\n"
        "Les modifications apportées par ces options visent la façon dont la valeur des dommages "
        "est déterminée.\n"
        "APPLICATION DE L'AVENANT\n"
        "Pour que les options s'appliquent au véhicule visé, elles doivent être écrites à la section "
        "« Conditions particulières » du contrat d'assurance.\n"
        "Sauf pour l'option 43C, l'assuré désigné peut choisir d'être indemnisé selon l'article 2 "
        "de la section « Déclarer un sinistre et faire une réclamation », sans que l'option choisie ne "
        "soit appliquée. Il doit alors en faire la demande à l'assureur.\n"
        "La franchise écrite à la section « Conditions particulières » pour le véhicule visé demeure à "
        "la charge de l'assuré désigné.\n"
        "VALEUR DES DOMMAGES EN CAS DE PERTE PARTIELLE\n"
        "Les options 43A et 43B visent à déterminer la valeur des dommages payable par l'assureur "
        "en cas de réparation ou de remplacement de pièces endommagées.\n"
        "Ces options modifient l'article 2.1 de la section « Déclarer un sinistre et faire une "
        "réclamation ». Par contre, les règles énoncées au premier paragraphe de l'article 2.1.1 "
        "de cette section demeurent applicables.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Option 43A – Perte partielle- Pièces neuves\n"
        "En cas de perte partielle, lorsque les pièces endommagées peuvent être réparées, la valeur "
        "des dommages est déterminée selon l'article 2.1 de la section « Déclarer un sinistre et faire "
        "une réclamation ». Par contre, aucune déduction pour la dépréciation ne s'applique.\n"
        "Si l'assureur détermine que des pièces endommagées ne peuvent pas être réparées et qu'elles "
        "doivent être remplacées, aucune déduction pour la dépréciation ne s'applique. De plus :\n"
        "\t la valeur des dommages est déterminée selon le coût des pièces d'origine du fabriquant neuves;\n"
        "\t si c'est une vitre qui est endommagée, l'assuré désigné peut demander qu'elle soit "
        "remplacée par une pièce d'origine du fabriquant neuve ou par une pièce similaire neuve.\n"
        "Si une pièce neuve n'est pas disponible ou qu'elle n'est plus fabriquée, l'assureur n'est tenu qu'au "
        "dernier prix courant de la pièce d'origine du fabricant neuve.\n"
        "Si les pièces endommagés ne sont ni réparées ni remplacées par des pièces neuves, l'option "
        "43A ne s'applique pas.\n"
        "Option 43B – Perte partielle – Indemnisation sans dépréciation\n"
        "En cas de perte partielle, la valeur des dommages est déterminée selon l'article 2.1 de "
        "la section « Déclarer un sinistre et faire une réclamation ». Par contre, si l'assureur "
        "détermine qu'une dépréciation doit être appliquée à la valeur des dommages, cette option "
        "couvrira cette dépréciation jusqu'à un montant maximum de  $ par sinistre.\n"
        "Par contre, si les pièces endommagés ne sont ni réparées ni remplacées, l'option 43B ne "
        "s'applique pas.\n"
        "VALEUR DES DOMMAGES EN CAS DE PERTE TOTALE OU RÉPUTÉE TOTALE\n"
        "Les options 43C, 43D, 43E et 43F visent à déterminer la valeur des dommages payable "
        "par l'assureur en cas de perte totale ou réputée totale. Ces options modifient l'article "
        "2.2 de la section « Déclarer un sinistre et faire une réclamation ».\n"
        "Option 43C – Perte totale - Valeur du véhicule convenue d'avance\n"
        "En cas de perte totale ou réputée totale, l'assuré désigné et l'assureur conviennent "
        "que la valeur du véhicule visé, au jour du sinistre, sera de :  $. "
        "(Il s'agit de la « valeur agréée ».)\n"
        "Option 43D – Perte totale - Indemnisation sans dépréciation\n"
        "En cas de perte totale ou réputée totale, la valeur des dommages équivaut au moins élevé "
        "des montants suivants :\n"
        "\t le prix payé pour le véhicule visé par l'assuré désigné qui en est le propriétaire réel;\n"
        "\t le prix courant du véhicule visé au jour de son achat;\n"
        "\t le prix auquel l'assureur peut, au jour du sinistre, acheter un véhicule neuf "
        "ayant des caractéristiques, équipements et accessoires semblables à ceux du véhicule visé.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        f"Option 43E – Perte totale - Indemnisation selon la valeur de remplacement du véhicule\n"
        f"En cas de perte totale ou réputée totale, l'assuré désigné a le choix entre les trois "
        f"types d'indemnisation suivants :\n"
        f"1. Remplacement par un véhicule neuf\n"
        f"Si le véhicule visé est remplacé par un véhicule neuf ayant les mêmes caractéristiques, "
        f"équipements et accessoires, la valeur des dommages équivaut au prix de ce dernier véhicule.\n"
        f"Si un tel véhicule n'est pas disponible et que le remplacement se fait plutôt par un véhicule "
        f"neuf ayant des caractéristiques, équipements et accessoires semblables, la valeur des dommages "
        f"équivaut au prix de ce dernier véhicule.\n"
        f"2. Remplacement par un véhicule usagé, ou par un véhicule neuf autre que celui du paragraphe 1.\n"
        f"Si le véhicule visé est remplacé par un véhicule usagé, ou par un véhicule neuf autre que celui du "
        f"paragraphe 1, la valeur des dommages équivaut au plus élevé des montants suivants :\n"
        f"\t le prix payé pour le véhicule visé par l'assuré désigné qui en est le propriétaire réel;\n"
        f"\t le prix du véhicule usagé ou neuf qui remplace le véhicule visé.\n"
        f"Par contre, la valeur des dommages ne peut jamais dépasser le prix que l'assureur aurait payé si "
        f"le paragraphe 1 avait été appliqué.\n"
        f"3. Aucun remplacement du véhicule visé\n"
        f"Si le véhicule visé n'est pas remplacé, la valeur des dommages équivaut au moins élevé des "
        f"montants suivants :\n"
        f"\t le prix payé pour le véhicule visé par l'assuré désigné qui en est le propriétaire réel;\n"
        f"\t le prix courant du véhicule visé au jour de son achat.\n"
        f"Par contre, la valeur des dommages ne peut jamais dépasser le prix que l'assureur aurait payé si "
        f"le paragraphe 1 avait été appliqué.\n"
        f"Option 43F – Perte totale - Augmentation de l'indemnité\n"
        f"En cas de perte totale ou réputée totale, la valeur des dommages équivaut :\n"
        f"\t A – au prix payé pour le véhicule visé par l'assuré désigné qui en est le propriétaire réel, "
        f"augmenté de % composé annuellement et calculé en proportion du nombre de jours "
        f"écoulés entre le  (date) et la date du sinistre;\n"
        f"ou\n"
        f"\t B – au montant de  $ augmenté de  % composé annuellement et calculé en "
        f"proportion du nombre de jours écoulés entre le  (date) et la date du sinistre;\n"
        f"ou\n"
        f"\t C – à la « valeur au jour du sinistre » du véhicule visé augmentée de  % composé "
        f"annuellement et calculée en proportion du nombre de jours écoulés entre le "
        f" (date) et la date du sinistre;\n"
        f"ou\n"
        f"\t D – à la « valeur au jour du sinistre » du véhicule visé augmentée de  $.\n"
        f"Dans tous les cas, la valeur des dommages ne peut jamais dépasser le prix auquel l'assureur peut, au "
        f"jour du sinistre, acheter un véhicule neuf ayant des caractéristiques, équipements et accessoires "
        f"semblables à ceux du véhicule visé.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "RÈGLE PARTICULIÈRE POUR LE VÉHICULE LOUÉ OU PRIS EN CRÉDIT-BAIL\n"
        "Lorsque le véhicule visé est loué ou pris en crédit-bail, que les options 43D, 43E ou 43F s'appliquent "
        "et que le propriétaire et le locataire ou crédit-preneur sont désignés au contrat d'assurance, seul le "
        "locataire ou le crédit-preneur a droit à la différence entre :\n"
        "\t la valeur du véhicule visé, déterminée selon l'option applicable; et\n"
        "\t la « valeur au jour du sinistre » du véhicule visé, déterminée selon l'article 2.2 de la "
        "section « Déclarer un sinistre et faire une réclamation ».\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq32_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 32\n"
        "Recreational-purpose vehicles\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N: \n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "Endorsement description\n"
        "This endorsement amends the insurance contract for a recreational-purpose motor vehicle that:\n"
        "\t is specifically designated in Item 3, ''Declarations'' of the insurance contract; or\n"
        "\t is one of the ''insured vehicles'' under the insurance contract.\n"
        "Recreational-purpose motor vehicle\n"
        "''Recreational-purpose motor vehicle'' means, among other things, any motor vehicle, "
        "whether commercially built or otherwise, of a type similar to:\n"
        "\t all-terrain vehicles;\n"
        "\t dune buggies;\n"
        "\t micro-cars;\n"
        "\t minibikes; and\n"
        "\t snowmobiles.\n"
        "Description of amendments\n"
        "1. Section A: Paragraph E, Article 2 entitled ''Insured vehicles'' is replaced with "
        "the following paragraph: ''E. Unless described in the ''Declarations'' section, any "
        "trailer (whether or not the named insured is the owner thereof) used with a "
        "recreational-purpose motor vehicle that is:\n"
        "\t of the same type as that described in the ''Declarations'' section; and\n"
        "\t covered by the insurance contract.\n"
        "2. General conditions: Paragraph (a), Article 7 entitled ''Prohibited use of "
        "insured vehicle'' is replaced with the following paragraph:\n"
        "''(a) When they are not legally authorized to drive;''\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "3. The expressions ''motor vehicle'' and ''motor vehicle used for personal purposes'' are replaced "
        "throughout the insurance contract with the following expression: "
        "''recreational-purpose motor vehicle of the same type as that described in "
        "the ‘Declarations' section.''\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq30_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 30\n"
        "Restriction des garanties pour certains équipements et matériel fixés au véhicule\n"
        "(Chapitre A)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Sous réserve de la Loi sur l'assurance automobile, cet avenant restreint les garanties du chapitre A du "
        "contrat d'assurance en y ajoutant l'exclusion suivante :\n"
        "\t Sont exclues les conséquences financières que peut subir la personne assurée lorsqu'elle "
        "est civilement responsable de dommages occasionnés par l'équipement ou le matériel suivant - ou "
        "leurs accessoires - pendant qu'ils sont fixés au véhicule visé :\n"
        "(Caractéristiques de l'équipement ou du matériel)\n"
        "Pour que l'exclusion ci-dessus s'applique :\n"
        "\t les dommages doivent être occasionnés pendant que l'équipement, le matériel ou leurs "
        "accessoires se trouvent sur les lieux de leur utilisation; et\n"
        "\t la responsabilité civile de la personne assurée doit découler du fait qu'elle est "
        "propriétaire de l'équipement, du matériel ou de leurs accessoires, ou du fait de leur usage ou de "
        "leur fonctionnement.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq34_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 34\n"
        "Assurance de personnes\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Prime d'assurance additionnelle à payer :\n"
        "\t Montants à payer :  \n"
        "\t Date limite pour payer : \n"
        "TABLE DES MATIERES\n"
        "DESCRIPTION DE L'AVENANT 2\n"
        "CONDITIONS D'APPLICATION 2\n"
        "PERSONNES ASSURÉES 2\n"
        "DIVISION APPLICABLE, MONTANTS MAXIMUMS ET PRIMES D'ASSURANCE 2\n"
        "DESCRIPTION DES DIVISIONS 3\n"
        "EXCLUSIONS 5\n"
        "QUOI FAIRE EN CAS D'ACCIDENT 6\n"
        "AUTOPSIE ET EXAMEN MÉDICAL 6\n"
        "DÉLAIS POUR LE PAIEMENT DES INDEMNITÉS 6\n"
        "DROITS DE L'ASSUREUR LIÉS AU PAIEMENT DES INDEMNITÉS OU DES FRAIS 7\n"
        "DÉFINITIONS 7\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "DESCRIPTION DE L'AVENANT\n"
        "Selon la division applicable, cet avenant prévoit le paiement d'indemnités lorsqu'un accident cause un "
        "dommage corporel à une personne assurée.\n"
        "Pour connaitre la division applicable, voir la section « Division applicable, montants "
        "maximums et primes d'assurance » de cet avenant.\n"
        "CONDITIONS D'APPLICATION\n"
        "1. Le dommage corporel doit avoir été causé directement du fait d'un véhicule automobile "
        "et indépendamment de toute autre cause.\n"
        "2. Les indemnités sont payables par accident et par personne assurée. Le fait qu'il y ait "
        "plusieurs véhicules assurés au contrat d'assurance ne permet pas de cumuler les indemnités.\n"
        "3. Seules les personnes qui respectent le contrat d'assurance et cet avenant peuvent "
        "bénéficier des avantages de cet avenant.\n"
        "PERSONNES ASSURÉES\n"
        "À la Division 1, l'expression « personne assurée » fait référence à l'assuré désigné, son conjoint "
        "et les enfants à charge de l'un ou de l'autre.\n"
        "À la Division 2, seul l'assuré désigné est une « personne assurée ».\n"
        "DIVISION APPLICABLE, MONTANTS MAXIMUMS ET PRIMES D'ASSURANCE\n"
        "Seules sont applicables les divisions et les subdivisions pour lesquelles un montant maximum "
        "ou une prime d'assurance est écrit au tableau ci-dessous, ou écrit spécifiquement pour cet "
        "avenant à la section « Conditions particulières » du contrat d'assurance :\n"
        "DIVISIONS | SUBDIVISIONS | MONTANTS MAXIMUMS | PRIMES D'ASSURANCE\n"
        "1 | A – Indemnités en cas de décès, B – Indemnités en cas de mutilation, C – Remboursement "
        "de frais médicaux | Capital assuré $, Capital assuré $, Montant maximum $ |  $,  $,  $"
        "2 Indemnités en cas d'incapacité totale | Montant maximum $, par semaine |  $|\n"
        "  |  | Total:|  $\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "DESCRIPTION DES DIVISIONS\n"
        "Division 1\n"
        "Indemnités en cas de décès et de mutilation et remboursement de frais médicaux\n"
        "Subdivision A - Indemnités en cas de décès\n"
        "Lorsqu'une personne assurée décède dans les 12 mois de l'accident qui a occasionné son décès, "
        "la Subdivision A prévoit le paiement des indemnités suivantes :\n"
        "\t Si c'est l'assuré désigné qui décède : 100% du capital assuré, auquel s'ajoute 10% "
        "du capital assuré par enfant à charge au décès de l'assuré désigné.\n"
        "\t Si c'est le conjoint de l'assuré désigné qui décède :  % du capital assuré, auquel "
        "s'ajoute 10% du capital assuré par enfant à charge au décès du conjoint.\n"
        "\t Si c'est un enfant à charge qui décède :  % du capital assuré.\n"
        "Les indemnités sont payables aux personnes suivantes :\n"
        "\t Si la personne assurée décédée laisse un conjoint, l'indemnité est payable à ce conjoint.\n"
        "\t Si la personne assurée décédée ne laisse pas de conjoint, l'indemnité est payable "
        "aux personnes à charge, en parts égales.\n"
        "\t Si la personne assurée décédée ne laisse pas de conjoint ni de personne à charge, "
        "l'indemnité est payable à sa succession.\n"
        "\t Si c'est un enfant à charge qui décède, l'indemnité est payable à l'assuré désigné.\n"
        "Si, en raison du même accident, une personne assurée subit une mutilation puis décède, "
        "les indemnités qui lui ont déjà été payées en vertu de la Subdivision B doivent être soustraites "
        "de l'indemnité payable en vertu de la Subdivision A.\n"
        "Subdivision B – Indemnités en cas de mutilation\n"
        "Lorsqu'une personne assurée perd la vue ou un membre de son corps dans les douze mois "
        "de l'accident qui a occasionné cette perte, la Subdivision B prévoit le paiement des indemnités "
        "suivantes :\n"
        "\t Perte des deux mains :100 % du capital assuré;\n"
        "\t Perte des deux pieds :100 % du capital assuré;\n"
        "\t Perte des deux yeux :100 % du capital assuré;\n"
        "\t Perte d'une main et d'un pied :100 % du capital assuré;\n"
        "\t Perte d'une main et d'un oeil :100 % du capital assuré;\n"
        "\t Perte d'un pied et d'un oeil :100 % du capital assuré;\n"
        "\t Perte d'un bras :75 % du capital assuré;\n"
        "\t Perte d'une jambe :75 % du capital assuré;\n"
        "\t Perte d'une main :50 % du capital assuré;\n"
        "\t Perte d'un pied :50 % du capital assuré;\n"
        "\t Perte d'un oeil :50 % du capital assuré.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "La perte d'un oeil signifie la perte totale et irrécouvrable de la vision.\n"
        "La perte d'un membre du corps signifie :\n"
        "\t Pour un bras :\n l'amputation au-dessus du coude.\n"
        "\t Pour une jambe : l'amputation au-dessus du genou.\n"
        "\t Pour une main : l'amputation au niveau du coude, au-dessus du poignet ou au niveau "
        "du poignet.\n"
        "\t Pour un pied : l'amputation au niveau du genou, au-dessus de la cheville ou au "
        "niveau de la cheville.\n"
        "Pour un même accident, les indemnités cumulées à la présente subdivision ne peuvent jamais "
        "dépasser 100% du capital assuré.\n"
        "Subdivision C – Remboursement de frais médicaux\n"
        "La Subdivision C prévoit le remboursement des frais raisonnablement engagés pour les services "
        "ou les soins suivants :\n"
        "\t services d'ambulance;\n"
        "\t soins chirurgicaux;\n"
        "\t soins dentaires;\n"
        "\t soins d'hôpitaux;\n"
        "\t soins d'infirmiers ou d'infirmières autorisés;\n"
        "\t soins médicaux.\n"
        "Ces frais doivent avoir été engagés dans les deux ans de l'accident qui les a occasionnés.\n"
        "Pour un même accident, le total des frais remboursés ne peut jamais dépasser le montant "
        "maximum fixé pour la présente subdivision.\n"
        "Exclusion\n"
        "Cette Subdivision C exclut la partie des frais médicaux visée par une autre assurance, "
        "gouvernementale ou privée, sauf si cette autre assurance est semblable à la présente.\n"
        "Division 2\n"
        "Indemnités en cas d'incapacité totale\n"
        "La Division 2 prévoit le paiement d'une indemnité de remplacement de revenu lorsque "
        "l'assuré désigné a une incapacité totale et ininterrompue en raison d'un accident. Cette "
        "indemnité est calculée à la semaine et n'est payable qu'en complément des montants suivants :\n"
        "\t ceux payables par la Société de l'assurance automobile du Québec en vertu de la "
        "Loi sur l'assurance automobile du Québec; et\n"
        "\t ceux payables en vertu de la Loi sur les accidents du travail et les maladies "
        "professionnelles et de la Loi sur l'indemnisation des victimes d'actes criminels.\n"
        "Conditions\n"
        "1. L'assuré désigné doit avoir un emploi au jour de l'accident. Il est réputé avoir un "
        "emploi dans l'une ou l'autre des situations suivantes :\n"
        "\t - il exerçait effectivement une activité à but lucratif;\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "- il était âgé de 21 ans à 65 ans, et dans les douze mois avant l'accident, il a exercé une "
        "activité à but lucratif pendant au moins six mois, avec ou sans interruption.\n"
        "2. L'incapacité doit se manifester dans les vingt jours de l'accident qui l'a occasionnée.\n"
        "3. L'incapacité empêche complètement l'assuré désigné d'exercer sa profession ou son "
        "occupation effective.\n"
        "Limitations\n"
        "Pour chaque accident :\n"
        "\t L'assuré désigné n'a droit à aucune indemnité pour les sept premiers jours de l'incapacité.\n"
        "\t Une fois ces sept jours expirés, il a droit aux indemnités pour un maximum de . semaines.\n"
        "\t Si l'incapacité persiste après ce nombre de semaines maximum, l'indemnité demeure "
        "payable pour une période supplémentaire maximum de . semaines, si :\n"
        "\t - la persistance est dûment attestée; et\n"
        "\t - l'incapacité empêche l'assuré désigné d'exercer toute activité à but lucratif de "
        "façon permanente.\n"
        "L'assuré désigné ne peut jamais recevoir une indemnité supérieure à la valeur en argent "
        "de toute activité à but lucratif qu'il exerçait avant l'accident.\n"
        "Si l'assuré désigné détient plusieurs assurances, l'indemnité se calcule de la façon suivante :\n"
        "Valeur en argent des activités/Total des indemnités payables pour l'incapacité "
        "en vertu de toutes les assurances X (fois) Indemnité payable à la Division 2 de cet avenant\n"
        "EXCLUSIONS\n"
        "Les exclusions suivantes s'appliquent :\n"
        "A. Les dommages corporels dus à une maladie, dans quelque mesure et à quelque titre que ce soit, "
        "sauf si cette maladie a été contractée directement du fait d'un accident couvert par cet avenant.\n"
        "B. Les dommages corporels subis par la personne assurée qui s'est suicidée ou qui a tenté de "
        "se suicider, peu importe son état mental, si cet avenant n'était pas en vigueur depuis au moins "
        "deux ans de façon ininterrompue.\n"
        "C. Qu'il y ait ou non déclaration de guerre, les dommages corporels occasionnés dans "
        "quelque mesure que ce soit par :\n"
        "\t les activités des forces armées engagées dans des hostilités;\n"
        "\t les bombardements;\n"
        "\t la force militaire;\n"
        "\t la guerre civile;\n"
        "\t l'insurrection;\n"
        "\t l'invasion;\n"
        "\t la rébellion;\n"
        "\t la révolution;\n"
        "\t l'usurpation de pouvoir.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "QUOI FAIRE EN CAS D'ACCIDENT\n"
        "Subdivision A - Indemnités en cas de décès\n"
        "En cas d'accident causant le décès, toute demande d'indemnité doit être envoyée à l'assureur, par écrit.\n"
        "Elle doit être accompagnée de preuves qui établissent, entre autres :\n"
        "\t le décès de la personne assurée;\n"
        "\t la cause de son décès;\n"
        "\t les droits des bénéficiaires de l'indemnité.\n"
        "Subdivisions B et C, et Division 2 - Indemnités en cas de mutilation, remboursement de frais "
        "médicaux et indemnités en cas d'incapacité totale\n"
        "En cas d'accident couvert par les Subdivisions B ou C, ou la Division 2, la personne assurée ou "
        "tout autre bénéficiaire d'une indemnité doit respecter les obligations énoncées ci-dessous. Elles "
        "peuvent le faire par elles-mêmes ou par un intermédiaire.\n"
        "Obligations à respecter :\n"
        "\t Dans les 30 jours de la date où cette personne a eu connaissance de l'accident, elle "
        "doit en informer l'assureur, par écrit.\n"
        "\t Dans les 90 jours de la date où cette personne a eu connaissance de l'accident, elle doit "
        "appuyer sa demande d'indemnité en donnant à l'assureur tous les renseignements auxquels il peut "
        "raisonnablement s'attendre sur les circonstances et sur l'étendue de l'accident.\n"
        "\t Si cette personne n'a pas respecté ces délais de 30 ou 90 jours, elle doit démontrer qu'il "
        "lui était impossible d'agir dans ces délais. Elle pourra quand même recevoir l'indemnité si "
        "elle respecte ses obligations dans l'année de l'accident.\n"
        "\t À la demande de l'assureur, cette personne doit fournir un certificat médical qui "
        "confirme les informations suivantes :\n"
        "- la cause et la nature des dommages corporels faisant l'objet de sa demande d'indemnité;\n"
        "- la durée de l'incapacité totale qui découle de ces dommages corporels.\n"
        "AUTOPSIE ET EXAMEN MÉDICAL\n"
        "Subdivision A - Indemnités en cas de décès\n"
        "Si une personne assurée décède, l'assureur peut faire pratiquer une autopsie. Il doit alors "
        "respecter les exigences du Code civil du Québec.\n"
        "Subdivisions B et C, et Division 2 - Indemnités en cas de mutilation, remboursement de frais médicaux "
        "et indemnités en cas d'incapacité totale\n"
        "L'assureur a le droit de faire passer un examen médical à la personne assurée dans un délai "
        "raisonnable, si la nature de l'invalidité ou de la perte le justifie.\n"
        "L'assureur peut lui faire passer cet examen aussi souvent qu'il en est raisonnable, tant et "
        "aussi longtemps que la demande d'indemnité est en suspens.\n"
        "La personne assurée est obligée de se soumettre à cet examen.\n"
        "DÉLAIS POUR LE PAIEMENT DES INDEMNITÉS\n"
        "Subdivision A - Indemnités en cas de décès\n"
        "L'assureur doit payer les indemnités dans les 30 jours où il reçoit les renseignements et les documents "
        "requis.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Subdivisions B et C - Indemnités en cas de mutilation et remboursement de frais médicaux\n"
        "L'assureur doit payer les indemnités ou rembourser les frais médicaux dans les 60 jours où il reçoit les "
        "renseignements et les documents requis.\n"
        "Division 2 - Indemnités en cas d'incapacité totale\n"
        "L'assureur doit payer la première indemnité dans les 30 jours où il reçoit les renseignements "
        "et les documents requis. À mesure que l'incapacité se prolonge, les paiements suivants se font "
        "à intervalles de 30 jours et moyennant justification tel que prévue à la section « Quoi "
        "faire en cas d'accident » de cet avenant.\n"
        "DROITS DE L'ASSUREUR LIÉS AU PAIEMENT DES INDEMNITÉS OU DES FRAIS\n"
        "L'assureur peut décider de payer une partie ou la totalité des indemnités ou des frais "
        "directement au bénéficiaire de l'assurance, ou aux personnes physiques ou morales qui ont "
        "fourni les soins ou les services médicaux. S'il reste un montant à payer, le total des indemnités "
        "ou des frais déjà payés est alors déduit du montant maximum de la subdivision visée.\n"
        "Avant de payer une indemnité :\n"
        "\t l'assureur peut exiger une quittance de la personne à qui il a payé une indemnité "
        "ou remboursé des frais.\n"
        "\t l'assureur peut exiger d'être subrogé dans les droits de recours du bénéficiaire "
        "contre la personne responsable des dommages corporels, jusqu'à concurrence du montant "
        "de l'indemnité qu'il a payée. Cela signifie que les droits de recours du bénéficiaire "
        "sont transférés à l'assureur.\n"
        "De plus, le paiement d'une indemnité ou le remboursement de frais médicaux ne constitue "
        "pas une admission de responsabilité quant à l'accident. Ce paiement ne peut donc pas être opposé "
        "à la personne assurée ou à l'assureur en matière de responsabilité civile.\n"
        "DÉFINITIONS\n"
        "Pour l'application de cet avenant, les définitions suivantes s'ajoutent à celles du contrat "
        "d'assurance ou les remplacent.\n"
        "ASSURÉ DÉSIGNÉ :\n"
        "\t l'expression « assuré désigné » fait référence à toute personne nommée à "
        "l'article 1 de la section « Conditions particulières » du contrat d'assurance;\n"
        "\t si la personne nommée à cet article 1 est une personne morale, une société ou une "
        "association, l'expression fait référence à chacun de ses employés, actionnaires, membres "
        "ou associés qui fait habituellement usage d'un véhicule désigné pour lequel une prime "
        "d'assurance ou un montant maximum est prévu spécifiquement pour cet avenant (voir l'article 4 "
        "de la section « Conditions particulières » du contrat d'assurance).\n"
        "DOMMAGE CORPOREL : tout dommage de nature physique, y compris le décès.\n"
        "ENFANT À CHARGE :\n"
        "\t tout enfant de moins de dix-huit ans qui est légalement et effectivement à la "
        "charge de l'assuré désigné ou de son conjoint;\n"
        "\t tout enfant de dix-huit ans ou plus qui a le même domicile que celui de l'assuré "
        "désigné et qui est entièrement à la charge de l'assuré désigné ou de son conjoint, ou des deux, "
        "en raison d'une infirmité mentale ou physique.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "PERSONNES À CHARGE : cette expression fait référence aux personnes suivantes :\n"
        "\t les enfants à charge; et\n"
        "\t le père ou la mère de la personne assurée qui est décédée, si les conditions suivantes "
        "sont respectées :\n"
        "\t le père ou la mère avait le même domicile que la personne assurée au jour de l'accident; et\n"
        "\t la personne assurée était le principal soutien financier du père ou de la mère jusqu'au "
        "jour de l'accident.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from unittest import TestCase

from risc_generator import ClaimsFaker


class ClaimsFakerTests(TestCase):
    n = 100000
    delta = 0.01  # A delta of 1%, meaning we accept a 1% deviation from the expected probabilities

    def test_whenClaimsFaker_thenAreInDeltaExpectedAverageNumberOfClaims(
        self,
    ):
        claims_faker = ClaimsFaker()
        number_of_years = 5

        payment_method = [claims_faker.claims(number_of_years=number_of_years) for _ in range(self.n)]

        # Expected number of claims in "number_of_years" (thus self.n for the number fo client times the number
        # of years we simulate claims)
        actual = sum(payment_method) / (self.n * number_of_years)
        expected = 0.0452

        self.assertAlmostEqual(actual, expected, delta=self.delta)



from faker import Faker

from ...domain.insuree import Insuree
from ...faker.insuree_faker.association_rebate_faker import (
    association_rebate_faker,
)
from ...faker.insuree_faker.claims_faker import ClaimsFaker
from ...faker.insuree_faker.driver_license_suspension_faker import (
    DriverLicenseSuspensionFaker,
)
from ...faker.insuree_faker.sex_faker import SexFaker


class InsureeFaker:
    def __init__(self, locale: str) -> None:
        """
        locale (str): The locale language setting to use for simulation. Can either be `'fr_CA'` or `'en_CA'`.
        """
        self.sex_faker = SexFaker(locale=locale)
        self.name_faker = Faker(locale=locale)
        self.date_of_birth_faker = Faker(locale=locale)
        self.address_faker = Faker(locale=locale)

        self.suspension_faker = DriverLicenseSuspensionFaker()
        self.claims_faker = ClaimsFaker()
        self.association_rebate_group_faker = association_rebate_faker

    def insuree(self) -> Insuree:
        """
        Method to fake an insuree.

        Return:
            An insuree in the locale language.
        """
        sex = self.sex_faker.gender()

        if sex == self.sex_faker.gender_1:
            # Gender 1 is either 'male' or 'homme' depending on the locale.
            name = self.name_faker.name_male()
        else:
            name = self.name_faker.name_female()

        date_of_birth = self.date_of_birth_faker.date_of_birth(minimum_age=16, maximum_age=95)

        address = self.address_faker.address()

        suspension = self.suspension_faker.suspension()

        number_of_claims_past_five_years = self.claims_faker.claims(number_of_years=5)

        association_rebate = self.association_rebate_group_faker()

        return Insuree(
            sex=sex,
            name=name,
            birth_date=date_of_birth,
            address=address,
            suspension=suspension,
            number_of_claims_past_years=number_of_claims_past_five_years,
            association_rebate=association_rebate,
        )



from unittest import TestCase

from risc_generator import (
    liability_coverage_amount_faker,
    b1_deductible_faker,
    b2_deductible_faker,
    b3_deductible_faker,
    b4_deductible_faker,
)


class ProtectionsAmountFakerTests(TestCase):
    n = 250000
    delta = 0.05  # A delta of 0.05, meaning we accept a 5% deviation from the expected probabilities

    def test_givenALiabilityCoverageAmountFaking_thenSimulationAreInDeltaExpectedProb(
        self,
    ):
        probs = [0.6, 0.4]
        coverages = [liability_coverage_amount_faker(liability_amount_probs=probs) == 1000000 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 1 million in liability coverage
        expected = probs[0]

        self.assertAlmostEqual(actual, expected, delta=self.delta)

        probs = [0.8, 0.2]
        coverages = [liability_coverage_amount_faker(liability_amount_probs=probs) == 1000000 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 1 million in liability coverage
        expected = probs[0]

        self.assertAlmostEqual(actual, expected, delta=self.delta)

    def test_givenAB1DeductibleAmountFaking_thenSimulationAreInDeltaExpectedProb(self):
        coverages = [b1_deductible_faker() == 250 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 250 of deductible
        expected_prod = 0.15
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b1_deductible_faker() == 500 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 500 of deductible
        expected_prod = 0.7
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b1_deductible_faker() == 1000 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 1000 of deductible
        expected_prod = 0.15
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

    def test_givenAB2DeductibleAmountFaking_thenSimulationAreInDeltaExpectedProb(self):
        coverages = [b2_deductible_faker() == 250 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 250 of deductible
        expected_prod = 0.20
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b2_deductible_faker() == 500 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 500 of deductible
        expected_prod = 0.65
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b2_deductible_faker() == 1000 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 1000 of deductible
        expected_prod = 0.15
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

    def test_givenAB3DeductibleAmountFaking_thenSimulationAreInDeltaExpectedProb(self):
        coverages = [b3_deductible_faker() == 250 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 250 of deductible
        expected_prod = 0.50
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b3_deductible_faker() == 500 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 500 of deductible
        expected_prod = 0.40
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b3_deductible_faker() == 1000 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 1000 of deductible
        expected_prod = 0.10
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

    def test_givenAB4DeductibleAmountFaking_thenSimulationAreInDeltaExpectedProb(self):
        coverages = [b4_deductible_faker() == 250 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 250 of deductible
        expected_prod = 0.55
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b4_deductible_faker() == 500 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 500 of deductible
        expected_prod = 0.40
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)

        coverages = [b4_deductible_faker() == 1000 for _ in range(self.n)]

        actual = sum(coverages) / self.n  # Prob of having 1000 of deductible
        expected_prod = 0.05
        self.assertAlmostEqual(actual, expected_prod, delta=self.delta)



from unittest import TestCase

from risc_generator import association_rebate_faker


class AssociationRebateFakerTests(TestCase):
    n = 250000
    delta = 0.05  # A delta of 0.05, meaning we accept a 5% deviation from the expected probabilities

    def test_whenAssociationRebateFaker_thenAreInDeltaExpectedRebateProb(
        self,
    ):
        payment_method = [association_rebate_faker() for _ in range(self.n)]

        actual = sum(payment_method) / self.n  # Prob of having a pre-authorize payment method
        expected = 0.05

        self.assertAlmostEqual(actual, expected, delta=self.delta)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq9_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 9\n"
        "Exclusion du risque maritime pour les véhicules amphibies\n"
        "Modifications\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat d'assurance.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Un véhicule amphibie est un véhicule conçu ou modifié pour:\n"
        "\t se déplacer sur la terre, et\n"
        "\t naviguer sur l'eau.\n"
        "Si le véhicule visé est un véhicule amphibie, cet avenant exclut du contrat d'assurance les sinistres "
        "qui surviennent pendant qu'il est mis à l'eau, y navigue ou en est retiré.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from unittest import TestCase
from unittest.mock import patch

from risc_generator import Insuree, InsureeFaker


class InsureeFakerTest(TestCase):
    def setUp(self) -> None:
        self.a_fr_locale = "fr_CA"
        self.a_en_locale = "en_CA"
        self.a_locale = self.a_en_locale

    def test_whenCreateAnInsuree_thenReturnAnInsuree(self):
        insuree_faker = InsureeFaker(self.a_locale)
        insuree = insuree_faker.insuree()
        self.assertIsInstance(insuree, Insuree)

    @patch("risc_generator.insuree_faker.SexFaker")
    def test_givenCreateMaleFakeInsuree_whenInsuree_thenReturnAMaleInsuree(self, sex_faker_mock):
        sex_faker_mock().gender.return_value = "male"
        insuree_faker = InsureeFaker(self.a_en_locale)

        insuree = insuree_faker.insuree()

        actual_sex = insuree.sex
        expected_sex = "male"

        self.assertEqual(actual_sex, expected_sex)

    @patch("risc_generator.insuree_faker.SexFaker")
    def test_givenCreateFemaleFakeInsuree_whenInsuree_thenReturnAFemaleInsuree(self, sex_faker_mock):
        sex_faker_mock().gender.return_value = "female"
        insuree_faker = InsureeFaker(self.a_en_locale)

        insuree = insuree_faker.insuree()

        actual_sex = insuree.sex
        expected_sex = "female"

        self.assertEqual(actual_sex, expected_sex)

    @patch("risc_generator.insuree_faker.SexFaker")
    def test_givenCreateHommeFakeInsuree_whenInsuree_thenReturnAHommeInsuree(self, sex_faker_mock):
        sex_faker_mock().gender.return_value = "homme"
        insuree_faker = InsureeFaker(self.a_en_locale)

        insuree = insuree_faker.insuree()

        actual_sex = insuree.sex
        expected_sex = "homme"

        self.assertEqual(actual_sex, expected_sex)

    @patch("risc_generator.insuree_faker.SexFaker")
    def test_givenCreateFemmeFakeInsuree_whenInsuree_thenReturnAFemmeInsuree(self, sex_faker_mock):
        sex_faker_mock().gender.return_value = "femme"
        insuree_faker = InsureeFaker(self.a_en_locale)

        insuree = insuree_faker.insuree()

        actual_sex = insuree.sex
        expected_sex = "femme"

        self.assertEqual(actual_sex, expected_sex)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq3_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 3\n"
        "Garantie « responsabilité civile » pour le véhicule d'un gouvernement canadien\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Prime d'assurance additionnelle à payer :\n"
        "\t Montants à payer :  \n"
        "\t Date limite pour payer :  \n"
        "Description de l'avenant\n"
        "Cet avenant étend la garantie « responsabilité civile » du contrat d'assurance au véhicule "
        "d'un gouvernement canadien dont la personne assurée a la garde ou sur lequel elle a un "
        "pouvoir de direction ou de gestion. Cet avenant couvre les conséquences financières que "
        "peut subir cette personne assurée lorsqu'elle est civilement responsable :\n"
        "1. Au chapitre A :\n"
        "\t d'un dommage causé à une autre personne par le véhicule ou ses équipements et accessoires.\n"
        "2. Au chapitre B :\n"
        "\t d'un dommage direct et accidentel causé au véhicule ou à ses équipements et accessoires; ou\n"
        "\t de la disparition du véhicule ou de ses équipements et accessoires.\n"
        "La responsabilité civile peut être contractuelle ou extracontractuelle.\n"
        "Personne assurée\n"
        "L'expression « personne assurée » utilisée dans cet avenant fait référence aux personnes suivantes :\n"
        "\t l'assuré désigné;\n"
        "\t son conjoint;\n"
        "\t toute personne qui conduit le véhicule ou en fait usage.\n"
        "Conditions d'application\n"
        "Cet avenant ne s'applique pas si la personne assurée a la garde de plusieurs véhicules "
        "d'un gouvernement canadien à la fois. Il ne s'applique pas non plus si elle a un pouvoir "
        "de direction ou de gestion sur plusieurs de ces véhicules à la fois.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Risques couverts, montant d'assurance, franchises et primes d'assurance\n"
        "Les risques couverts par cet avenant sont ceux pour lesquels un montant d'assurance, une franchise "
        "ou une prime d'assurance est écrit au tableau ci-dessous, ou écrit spécifiquement pour cet avenant "
        "à la section « Conditions particulières » du contrat d'assurance.\n"
        "GARANTIES | RISQUES | MONTANT D'ASSURANCE ET FRANCHISES | PRIMES D'ASSURANCE\n"
        "Chapitre A : Responsabilité civile | Dommages matériels ou dommages corporels causés à "
        "d'autres personnes | Montant d'assurance :  $ |  $\n"
        "Chapitre B : Dommages aux véhicules assurés | Protection 1 : « Tous risques », Protection 2 : "
        "Risques de collision et de renversement, Protection 3 : Tous les risques sauf collision ou renversement, "
        "Protection 4 : Risques spécifiques | Franchise par sinistre : $ $ $ $ | $ $ $ $\n"
        "  |  | Total: | $\n"
        "Pour le chapitre B, la garantie de cet avenant s'applique pour un montant maximum de  $ "
        "par sinistre, auquel s'ajoutent les frais, les dépens et les intérêts qui découlent d'une poursuite.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from risc_generator import Protections, FPQ1Contract, Premium
from tests.domain.contract.contract_base import ContractTestBase

include = "include"


class ContractBillTest(ContractTestBase):
    def setUp(self) -> None:
        self.a_protections = Protections(protections=self.a_dict_of_protections)

    def test_givenAFRContract_whenGenerateBillText_thenReturnProperFRText(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityCoveragePremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33ACoveragePremium": a_premium_amount,
            "FAQ34ACoveragePremium": a_premium_amount,
            "FAQ43ACoveragePremium": a_premium_amount,
        }
        a_premium = Premium(**protection_premiums)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method="facture",  # FR bill payment method
            protections=self.a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_bill_text()

        a_client_number = "1234567"
        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"
        a_premium_total = "7"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "FACTURATION - Reçu officiel\n"
            "Assurance des particuliers\n"
            f"Numéro client : {a_client_number}\n"
            f"Nom et adresse de l'assuré\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"Nom du payeur : {a_insuree_name}\n"
            f"ÉTAT DE VOTRE COMPTE - MODE DE PAIEMENT COMPTANT SEULEMENT\n"
            f"CATÉGORIE D'ASSURANCE | N POLICE | TRANSACTION ACTUELLE | PRISE D'EFFET | TYPE DE TRANSACTION | "
            f"MONTANT INCLUANT TAXE | SOLDE DE LA POLICE | TOTAL\n"
            f"Automobile | 001 | | | SOLDE AVANT TRANSACTION | | {a_premium_total}$ | {a_premium_total}$\n"
            f"À payer-terme en vigueur {a_premium_total}$\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractNoBill_whenGenerateBillText_thenReturnEmptyText(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityCoveragePremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33ACoveragePremium": a_premium_amount,
            "FAQ34ACoveragePremium": a_premium_amount,
            "FAQ43ACoveragePremium": a_premium_amount,
        }
        a_premium = Premium(**protection_premiums)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_bill_text()

        expected = ""

        self.assertEqual(expected, actual)

    def test_givenAENContract_whenGenerateBillText_thenReturnProperENText(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityCoveragePremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33ACoveragePremium": a_premium_amount,
            "FAQ34ACoveragePremium": a_premium_amount,
            "FAQ43ACoveragePremium": a_premium_amount,
        }
        a_premium = Premium(**protection_premiums)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_bill_payment_method,
            protections=self.a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_bill_text()

        a_client_number = "1234567"
        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"
        a_premium_total = "7"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "INVOICING - Official receipt\n"
            "Personal Insurance\n"
            f"Customer number: {a_client_number}\n"
            f"Name and address of the insured\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            f"Payer's name: {a_insuree_name}\n"
            f"YOUR ACCOUNT STATUS - CASH ONLY\n"
            f"CLASS OF INSURANCE | POLICY NUMBER | CURRENT TRANSACTION | EFFECTIVE DATE | TRANSACTION TYPE "
            f"AMOUNT INCLUDING TAX | POLICY BALANCE | TOTAL\n"
            f"Automotive | 001 | | | BALANCE BEFORE TRANSACTION | | {a_premium_total}$ | {a_premium_total}$\n"
            f"Payable-Term in effect {a_premium_total}$\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractNoBill_whenGenerateBillText_thenReturnEmptyText(self):
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityCoveragePremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33ACoveragePremium": a_premium_amount,
            "FAQ34ACoveragePremium": a_premium_amount,
            "FAQ43ACoveragePremium": a_premium_amount,
        }
        a_premium = Premium(**protection_premiums)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_bill_text()

        expected = ""

        self.assertEqual(expected, actual)



from ..config import FAKE_INSURER_WEBSITE, FAKE_INSURER_NAME, NEW_PAGE_TAG, FAKE_MENTAL_HEALTH_PROGRAM_NAME


def generate_client_advantages_text(self) -> str:
    text = (
        "AVANTAGES CLIENT\n"
        f"Suivez-nous\n\n"
        f"Additionez les économies en regroupant "
        f"vos assurances chez nous! Visitez "
        f"{FAKE_INSURER_WEBSITE} pour plus de détails.\n"
        f"Suivez-nous sur Facebook pour être "
        f"informé de nos concours et promotions!\n"
        f"(Facebook.com/{FAKE_INSURER_NAME}).\n"
        f"Visitez notre blogue {FAKE_INSURER_WEBSITE} pour "
        f"obtenir une foule de rensignements utiles "
        f"liés au domaine de l'assurance et des "
        f"services financiers.\n"
    )

    text += "Économies dont vous bénéficiez\n\n"
    rebate_text = ""

    # No multi-product case to deal with as the dataset is composed just of new customer with a single car on
    # the contract and single product.
    if self.payment_method == "pre-authorize":
        rebate_text += "Rabais pour avoir choisi le Paiement pré-autorisé comme mode de paiement de votre prime\n"
    if self.insuree.insuree_is_in_a_rebate_group():
        rebate_text += "Rabais parce que vous êtes membre/client d'une association\n"
    if self.vehicle.is_electric():
        rebate_text += "Rabais pour un véhicule électrique\n"
    if rebate_text == "":
        # Case for no rebate
        rebate_text += f"Regroupez vos assurances à {FAKE_INSURER_NAME} et bénéficiez d'économies substantielles\n"

    text += rebate_text

    text += (
        "Rappel des avantages clients\n\n"
        "Votre assurance auto\n"
        "\tTarification privilégiée pour les conducteurs sans réclamation.\n"
        "\tCouverture automatique si vous conduisez un véhicule loué ou emprunté; au "
        "Canada ou aux États-Unis (la limite admissible est précisée à la garantie "
        "F.A.Q. N 27 de votre police d'assurance automobile).\n"
        "\tLocation d'un véhicule de remplacement en cas de sinistre couvert : l'avenant "
        "F.A.Q. N 20a de votre police d'assurance couvre notamment les frais de location d'une automobile et, "
        "en cours de voyage, les frais supplémentaires de subsistance encourus à la suite d'un sinistre couvert"
        " vous privant de votre véhicule pendant la réparation. Les montants disponibles sont indiqués dans "
        "votre police d'assurance.\n"
        "\t0 $ de franchise en cas de délit de fuite, de perte totale ou de réparation de pare-brise.\n"
        "\tObtenez 10 % de rabais supplémentaire en assurant un autre véhicule "
        "(auto, moto, motoneige, quad, bateau, caravane ou autocaravane) et votre résidence chez nous.\n"
        "\tObtenez 5 % de rabais supplémentaire en assurant aussi votre résidence chez nous.\n"
    )

    text += (
        "Votre assistance juridique\n\n"
        "\tService téléphonique de renseignements juridiques inclus gratuitement avec toute assurance.\n"
        "\tQuel que soit le problème, nos avocats membres du Barreau du Québec sont à l'écoute pour vous "
        "informer précisément sur vos droits. Confidentialité assurée.\n"
        "\tMunissez-vous d'une protection légale complète à peu de frais en ajoutant"
        " l'assurance protection juridique à vos protections. Appelez-nous pour obtenir les détails et profiter "
        "de ce privilège!\n"
    )

    text += NEW_PAGE_TAG + "\n"
    text += "AVANTAGES CLIENT\n"

    text += (
        "Rappel des avantages clients\n\n"
        "Indemnisation\n"
        "\tService d'indemnisation rapide et équitable, disponible 24 h sur 24, 7 jours sur 7.\n"
        "\tSi un sinistre touche plus d'un produit assuré avec nous, vous n'aurez qu'une seule "
        "franchise à payer, soit la plus élevée des deux.\n"
        "\tAfin de protéger votre santé psychologique, nous sommes heureux de vous offrir "
        "gratuitement les services professionnels pour vous aider à surmonter les difficultés découlant d'un "
        "sinistre.\n"
        "Totalement confidentiel, ce service est offert à tous nos assurés dans les 12 mois suivant un sinistre "
        f"couvert touchant une police d'assurance des particuliers. Pour tous les détails, visitez "
        f"{FAKE_INSURER_NAME}.ca/{FAKE_MENTAL_HEALTH_PROGRAM_NAME}.\n"
    )

    text += NEW_PAGE_TAG + "\n"

    return text



from .general_header import generate_general_pages_header_text
from ..config import GENERAL_FAKE_PHONE_NUMBER_1_800, GENERAL_FAKE_PHONE_NUMBER, NEW_PAGE_TAG


def generate_cancellation_of_policy_text(self):
    general_header_text = generate_general_pages_header_text(policy_number=self.policy_number)
    general_header_text += "CANCELLATION OF POLICY\n"

    text = (
        f"{general_header_text}"
        "If your policy covers several vehicles and coverage for only one of them must be cancelled, do not use this "
        "form.\n"
        "Please contact Customer Service at the number listed below.\n"
        f"BEFORE CANCELLING contact Customer Service at {GENERAL_FAKE_PHONE_NUMBER} or "
        f"{GENERAL_FAKE_PHONE_NUMBER_1_800}, this could avoid you losing some of your advantages.\n"
        f"I hereby request the full cancellation of policy number {self.policy_number}, "
        "its endorsements, its renewals and if applicable, the reimbursement of any unearned premium effective: "
        "______________      ______________      ______________\n"
        "(Year) (Month) (Day)\n"
        "Signature :  ____________________________________________________________   "
        "Date : _____________     _____________     _____________\n"
        "(Insured) (Year) (Month) (Day)\n"
        "Signature :  ____________________________________________________________   "
        "Date : _____________     _____________     _____________\n"
        "(Additional Insured) (Year) (Month) (Day)\n"
        "Signature :  ____________________________________________________________   "
        "Date : _____________     _____________     _____________\n "
        "(Additional Insured) (Year) (Month) (Day)\n"
        "Reason of cancellation:  _____________________________________________  "
        "New insurer:  ________________________________________\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



import datetime
from unittest import TestCase
from unittest.mock import MagicMock

from risc_generator import Protections, Premium
from risc_generator.domain.insuree import Insuree
from risc_generator.domain.vehicle import Vehicle

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractTestBase(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.a_client_number = "1234567"

        cls.a_contract_start_date = datetime.date(year=2022, month=2, day=1)

        cls.a_sex = "male"
        cls.a_name = "A Name"
        cls.a_date_of_birth = datetime.date(year=2020, month=1, day=1)
        cls.an_address = "An address\nA city, QC, A1A 1A1"
        cls.a_suspension = False
        cls.a_number_of_claims_past_years = 1
        cls.a_association_rebate = False
        cls.a_insuree = Insuree(
            sex=cls.a_sex,
            name=cls.a_name,
            birth_date=cls.a_date_of_birth,
            address=cls.an_address,
            suspension=cls.a_suspension,
            number_of_claims_past_years=cls.a_number_of_claims_past_years,
            association_rebate=cls.a_association_rebate,
        )

        cls.a_year = 2020
        cls.a_maker = "AMaker"
        cls.a_model = "AModel"
        cls.a_vin = "1234567890"
        cls.a_purchase_condition = "used"
        cls.a_vehicle = Vehicle(
            year=cls.a_year,
            make=cls.a_maker,
            model=cls.a_model,
            vin=cls.a_vin,
            purchase_condition=cls.a_purchase_condition,
            electric_vehicle=False,
        )

        cls.a_pre_authorize_payment_method = "pre-authorize"
        cls.a_bill_payment_method = "bill"

        cls.a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ27ACoverage": INCLUDE,
        }

        cls.a_fr_locale = "fr_CA"
        cls.a_en_locale = "en_CA"
        cls.a_locale = cls.a_fr_locale

        cls.a_premium_mock = MagicMock()

        cls.a_creditor = "A Bank An Address for The Bank"

        cls.a_protections = Protections(protections=cls.a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        cls.a_premium = Premium(**protection_premiums)



# pylint: disable=too-many-locals

from risc_generator import Protections, FPQ1Contract, Premium
from risc_generator.content import fr
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractDeclarationTest(ContractTestBase):
    def test_givenAFRContractLiabilityOnly_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        protection_premiums = {"LiabilityCoveragePremium": a_liability_coverage_premium}
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": EXCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Non couvert\n"
            "Protection 2 : Risques de collision et de renversement Non couvert\n"
            "Protection 3 : Tous les risques sauf collision ou renversement Non couvert\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractLiabilityWithB1_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b1_premium = 2
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB1CoveragePremium": a_b1_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": 250,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": EXCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques - Franchise 250$ par sinistre 2$\n"
            "Protection 2 : Risques de collision et de renversement Non couvert\n"
            "Protection 3 : Tous les risques sauf collision ou renversement Non couvert\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractLiabilityWithB2B3_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractLiabilityWithB2B4_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b4_premium = 3
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB4CoveragePremium": a_b4_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement Non couvert\n"
            "Protection 4 : Risques spécifiques - Franchise 250$ par sinistre 3$\n"
            f"{a_new_page}\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ33_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ33CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ33Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"
        a_roadside_program_name = "ABC"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 33 - Assurance pour les frais d'assistance routière\n"
            f"{a_roadside_program_name} Programme d'assistance routière, disponible 24 heures sur 24, 7 jours sur 7\n"
            f"Rayon maximum 50 km -\n"
            f"En cas de panne, la distance de remorquage est haussée à 100 km dans le parc des Laurentides, "
            f"150 km dans le parc de la Vérendrye et 100 km dans le parc national de la Gaspésie\n"
            f"Nombre d'événements maximum par année d'assurance 4 4$\n"
            f"Limite par dépannage 60$(1) - Pour un remorquage ou un véhicule enlisé, cette limite est haussée à "
            f"100$(1)\n"
            f"Limite par année d'assurance 400$(1)\n"
            f"(1) Ces limites s'appliquent uniquement lorsque, de façon exceptionnelle, le service de dépannage ou de "
            f"remorquage n'est pas disponible par le programme d'assistance routière\n"
            f"Consultez votre brochure pour connaître tous les services additionnels qui vous sont offerts\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ20A_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ20ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ20ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 20a - Frais de déplacement (formule étendue) (Chapitre B) 4$\n"
            "Frais de déplacement 50$ par jour, limitation totale de 1500$ par sinistre\n"
            "Autres frais couverts au cours d'un voyage - 750$\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ20_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ20CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ20Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 20 - Frais de déplacement (formule étendue) (Chapitre B) 4$\n"
            "Frais de déplacement 50$ par jour, limitation totale de 1500$ par sinistre\n"
            "Autres frais couverts au cours d'un voyage - 750$\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ34_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ34CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ34Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 34 - Assurance de personnes : 4$\n"
            "Division 1 : Subdivision A et B - Indemnités en cas de décès - mutilation : 20000$ capital assuré\n"
            "Subdivision C - Remboursement de frais médicaux : 2000$ par personne\n"
            "Division 2 : Incapacité totale : non couvert\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ34AB_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ34ABCoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ34ABCoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 34ab - Assurance de personnes : 4$\n"
            "Division 1 : Subdivision A et B - Indemnités en cas de décès - mutilation : 20000$ capital assuré\n"
            "Subdivision C - Remboursement de frais médicaux : 2000$ par personne\n"
            "Division 2 : Incapacité totale : non couvert\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ43_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ43CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ43Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 43a - Modification à l'indemnisation (Chapitre B) - Perte partielle - Pièces neuves"
            " 1$\n"
            "F.A.Q. N 43e - Modification à l'indemnisation (Chapitre B) - Perte totale - Indemnisation selon la "
            "valeur de remplacement du véhicule 3$\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ2_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ2CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ2Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 2 - Conduite de véhicules dont l'assuré désigné n'est pas propriétaire par "
            "des conducteurs désignés (Chapitre A) Incluse\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ5A_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ5ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ5ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 5a - Véhicules loués ou sous contrat de location Incluse\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ23A_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ23ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ23ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 23a - Véhicules sous financement Incluse\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ27_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ27CoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ27Coverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 27 - Responsabilité civile du fait de dommages causés à des véhicules "
            "dont l'assuré désigné n'est pas propriétaire (incluant les véhicules fournis par un employeur) "
            "(Chapitre A) - 75000$\n"
            "Véhicules assimilables au véhicule désigné, aux caravanes ou remorques utilitaires Chapitre "
            "B2 - Franchise 250$ et chapitre B3 - Franchise 50$ Incluse\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractWithEndorsementFAQ27a_whenGenerateItem4Text_thenReturnProperFRText(self):
        a_liability_coverage_premium = 1
        a_b2_premium = 2
        a_b3_premium = 3
        a_faq_premium = 4
        protection_premiums = {
            "LiabilityCoveragePremium": a_liability_coverage_premium,
            "PropertyDamageB2CoveragePremium": a_b2_premium,
            "PropertyDamageB3CoveragePremium": a_b3_premium,
            "FAQ27ACoveragePremium": a_faq_premium,
        }
        a_premium = Premium(**protection_premiums)

        a_dict_of_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ27ACoverage": INCLUDE,
        }
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_4_text(
            protections=fpq1_contract.protections, premium=fpq1_contract.premium
        )

        a_insurer_name = "Assureur"
        fidelity_program_name = "FIDÉLITÉ"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "Article 4. GARANTIES / RISQUES : Les risques couverts par le contrat d'assurance sont ceux "
            "pour lesquels un montant d'assurance, une franchise ou une prime d'assurance est écrit au tableau "
            "ci-dessous. Ils sont couverts aux conditions énoncées dans le contrat d'assurance.\n"
            "PRIME\n"
            "\t Chapitre A - RESPONSABILITÉ CIVILE -\n"
            "Dommages matériels ou dommages corporels causés à d'autres personnes - Montant : 1000000$ "
            f"{a_liability_coverage_premium}$\n"
            "\t Chapitre B - DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "Protection 1 : Tous risques Incluse\n"
            "Protection 2 : Risques de collision et de renversement - Franchise 250$ par sinistre 2$\n"
            "Protection 3 : Tous les risques sauf collision ou renversement - Franchise 250$ par sinistre 3$\n"
            "Protection 4 : Risques spécifiques Non couvert\n"
            f"{a_new_page}\n"
            "Avenant(s) : F.A.Q. N 27a - Responsabilité civile du fait de dommages causés à des véhicules "
            "dont l'assuré désigné n'est pas propriétaire (incluant les véhicules fournis par un employeur) "
            "(Chapitre A) - 75000$\n"
            "Véhicules assimilables au véhicule désigné, aux caravanes ou remorques utilitaires Chapitre "
            "B2 - Franchise 250$ et chapitre B3 - Franchise 50$ Incluse\n"
            f"\t Programme {fidelity_program_name}\n"
            f"F.A.Q. N 41\n"
            f"- Modification aux franchises (Chapitre B) : Incluse\n"
            f"Cet avenant apporte les modifications suivantes aux franchises du chapitre B :\n"
            f"\t - Suppression de la franchise, pour un délit de fuite s'il y a eu un rapport de police.\n"
            f"\t - Suppression de la franchise, pour une perte totale.\n"
            f"\t - Suppression de la franchise, pour une réparation de pare-brise sauf lors de son remplacement.\n"
            f"\t - Si vous êtes victime d'un sinistre qui affecte deux produits ou plus, assurés à "
            f"{a_insurer_name}, vous n'aurez qu'une seule franchise à payer, soit la plus élevée.\n"
            "Date d'échéance de prime : selon le mode de paiement convenu\n"
        )

        self.assertEqual(expected, actual)



from ..config import (
    CLAIMS_FAKE_PHONE_NUMBER,
    CLAIMS_FAKE_PHONE_NUMBER_1_800,
    GENERAL_FAKE_PHONE_NUMBER_1_800,
    FAKE_INSURER_WEBSITE,
    FAKE_INSURER_NAME,
    NEW_PAGE_TAG,
)
from ..fr_en.back_of_certificates import back_of_certificates_text


def generate_certificates_text(self) -> str:
    text = (
        "The insured vehicle's insurance certificates can now be downloaded into the Apple Wallet application. "
        "Details on the back.\n"
    )

    if self.protections.is_protected("FAQ27Coverage") or self.protections.is_protected("FAQ27aCoverage"):
        text += (
            f"Q.E.F. N 27 : YOUR RENTAL VEHICLE INSURANCE CERTIFICATE\n"
            f"The next time you rent a car, show this certificate to the rental agent to avoid paying additional "
            f"insurance fees. It's valid in Canada and the United States up to a maximum of "
            f"Details on back of the certificate.\n"
            f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
            f"{self.policy_number}\n"
            f"{self.insuree.name}\n"
            f"{self.insuree.address}\n"
            f"DURÉE DU CONTRAT DU "
            f"{self.contract_start_date} AU {self.contract_end_date}\n"
            f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
            f"IMPORTANT\n"
            f"AU BESOIN, VEUILLEZ PRÉSENTER CE CERTIFICAT À VOTRE AGENCE DE LOCATION DE VOITURES\n"
            f"IF NECESSARY, PLEASE PRESENT THIS CERTIFICATE TO YOUR AUTOMOBILE RENTAL AGENCY\n"
            f"Need to file an insurance claim?\n"
            f"Contact us.\n"
            f"Claims Department open 24/7\n"
            f"Local: {CLAIMS_FAKE_PHONE_NUMBER}\n"
            f"Toll free: {CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
            f"Customer Service\n"
            f"Toll free:{GENERAL_FAKE_PHONE_NUMBER_1_800}\n\n"
        )

    text += (
        f"HERE ARE YOUR TWO INSURANCE CERTIFICATES FOR THE INSURED VEHICLE\n"
        f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
        f"{self.policy_number}\n"
        f"{self.insuree.name}\n"
        f"{self.insuree.address}\n"
        f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
        f"{self.vehicle.format_vehicle_complete_details()}\n"
        f"DURÉE DU CONTRAT DU "
        f"{self.contract_start_date} AU {self.contract_end_date}\n"
        f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
        f"Need to file an insurance claim?\n"
        f"Contact us.\n"
        f"Claims Department open 24/7\n"
        f"Local: {CLAIMS_FAKE_PHONE_NUMBER}\n"
        f"Toll free: {CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
        f"Customer Service\n"
        f"Toll free:{GENERAL_FAKE_PHONE_NUMBER_1_800}\n"
        f"NUMÉRO DE POLICE / POLICY NUMBER\n\n"
        f"{self.policy_number}\n"
        f"{self.insuree.name}\n"
        f"{self.insuree.address}\n"
        f"VÉHICULE ASSURÉ / INSURED VEHICLE\n"
        f"{self.vehicle.format_vehicle_complete_details()}\n"
        f"DURÉE DU CONTRAT DU "
        f"{self.contract_start_date} AU {self.contract_end_date}\n"
        f"POLICY PERIOD FROM An/Yr M J/D TO An/Yr M J/D\n"
        f"Need to file an insurance claim?\n"
        f"Contact us.\n"
        f"Claims Department open 24/7\n"
        f"Local: {CLAIMS_FAKE_PHONE_NUMBER}\n"
        f"Toll free: {CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
        f"Customer Service\n"
        f"Toll free:{GENERAL_FAKE_PHONE_NUMBER_1_800}\n\n"
        f"{NEW_PAGE_TAG}\n"
        "CANADA INTER-PROVINCE\n"
        "MOTOR VEHICLE LIABILITY INSURANCE CARD\n"
        "Keep with your driver's licence\n"
        f"To download your certificate into the Apple Wallet application (available on iPhone only):\n"
        f"\t1. Log in to the Client Centre from your phone at {FAKE_INSURER_WEBSITE}/clientcentre.\n"
        f"\t2. In the Home, auto and leisure vehicles section, click on ''Get an insurance certificate'', then"
        f"''Add to Apple Wallet''.\n"
        f"You will have to update your certificates at each renewal of, or change to, your insurance policy by "
        f"downloading it again to the Apple Wallet application.\n"
        f"{FAKE_INSURER_NAME} cannot guarantee that the electronic version of the certificates will be accepted by "
        f"all police forces. Make sure you always have a paper version on hand.\n\n"
    )

    if self.protections.is_protected("FAQ27Coverage") or self.protections.is_protected("FAQ27aCoverage"):
        text += (
            f"CERTIFICAT D'ASSURANCE POUR LOCATION DE VOITURE\n"
            f"(Valide au Canada et aux États-Unis d'Amérique seulement)\n"
            f"Application particulière de l'avenant F.A.Q. 27 : Responsabilité civile du fait de dommages causés à "
            f"des véhicules dont l'assuré désigné n'est pas propriétaire (incluant les véhicules fournis par un "
            f"employeur).\n"
            f"En cas de vol ou d'endommagement d'un véhicule de location, {FAKE_INSURER_NAME} s'engage "
            f"à rembourser, à l'Assuré ou au propriétaire du véhicule, le montant des dommages réellement encourus "
            f"et dont l'Assuré ou son conjoint s'est rendu responsable en vertu d'un contrat de location, le tout "
            f"jusqu'à concurrence de : 75 000 $\n\n"
            f"CAR RENTAL INSURANCE CERTIFICATE\n"
            f"(Valid only in Canada and the United States of America)\n"
            f"Speciﬁc application of Q.E.F. endorsement B 27 : Civil liability resulting from damage caused to "
            f"vehicles of which named insured is not owner (including vehicles provided by an employer).\n"
            f"In the event of theft or damage to a rental vehicle, {FAKE_INSURER_NAME} "
            f"agrees to refund the Insured or the owner of the vehicle, the amount of damage actually "
            f"incurred and for which the Insured or his/her spouse assumes liability under a rental "
            f"contract, in all up to: $75,000\n"
        )

    # Two certificates
    text += back_of_certificates_text
    text += back_of_certificates_text
    text += NEW_PAGE_TAG + "\n"

    return text



from unittest import TestCase
from unittest.mock import patch

from risc_generator import FinancingFaker


class FinancingFakerTest(TestCase):
    @patch("risc_generator.financing_faker.Faker")
    @patch("risc_generator.financing_faker.choice")
    def test_whenFinancingFaker_thenReturnFinancingDetails(self, bank_faker, address_faker):
        bank_faker.return_value = "A Bank"
        address_faker().address.return_value = "An Address"
        financing_faker = FinancingFaker(locale="fr_CA")

        actual = financing_faker.financing()
        expected = "A Bank An Address"
        self.assertEqual(expected, actual)
